"""Sequence based similarity measure"""

from py_stringmatching.similarity_measure.similarity_measure import \
                                                        SimilarityMeasure

class SequenceSimilarityMeasure(SimilarityMeasure):
    pass
